/* 
 * FILE --- fuzzy_sys_ex2.cc
 *
 * PURPOSE --- To provide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
 * n-input 1-output Takagi-Sugeno (TS) fuzzy system, FUZZY_TS_SYS.
 *
 * Implements the main function of a class for performing a simulation of
 * a simple vehicle with point mass dynamics controlled by a TS fuzzy controller.
 * The fuzzy controller is implemented as a part of the class for the vehicle
 * dynamics - VEH_SIM_FUZZY_TS. 
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>

#include "veh_sim_fuzzy_ts.hh"


void main(void) 
{
  int ii;  // Index variales for the loops
  double Ts = 0.01;             // Sampling time
  double y[3] = {-0.9*45.0,45.0,0.0};  // Initial conditions    
  double t0 = 0.0;              // Initial time 
  double tmax = 200.0;              // Total Simulation period

  // TS coefficients
  double **ts_coeff;
 
  // SISO fuzzy controller gains
  double input_gain = 1/15.0, output_gain = 2000.0;

  // # of simulation steps to be performed by the RK4 algorith
  // in one sampling interval
  int sim_per_cycle = 5;  

  // Create the vehicle object
  VEH_SIM_FUZZY_TS my_vehicle(Ts,sim_per_cycle,t0,(double *)y);

  FILE *fp;  // To save data

  if (!(fp = fopen("veh_fuzzy_ts_out1.dat","w"))) 
    {
      printf("Can't open file.\n");
      exit(1);
    }

  if ((ts_coeff = new double*[my_vehicle.fuzzy_ts_sys->get_num_of_rules()]) == NULL)
    printf("Memory error!!!!\n");

  for(ii = 0; ii < my_vehicle.fuzzy_ts_sys->get_num_of_rules(); ii++)
    {
      if ((ts_coeff[ii] = new double[my_vehicle.fuzzy_ts_sys->get_num_of_ts_inputs()+1]) == NULL)
        printf("Memory error!!!!\n");
    }

  // Set the rules uniformly distributed (I use this so that to compare 
  // the result with that of the standard fuzzy system)
  for(ii = 0; ii < my_vehicle.fuzzy_ts_sys->get_num_of_rules(); ii++)
    {
      ts_coeff[ii][0] = (2.0*ii)/6.0 - 1.0; 
    }
  
  // Set the TS fuzzy controller input gains
  my_vehicle.fuzzy_ts_sys->set_input_gains(&input_gain);

  // Set the coefficients to the TS inputs of the fuzzy controller
  my_vehicle.fuzzy_ts_sys->set_ts_coeff((const double **)ts_coeff);

  // Set the output gain for the TS fuzzy controller
  my_vehicle.output_gain = output_gain;

  // Simulate For tmax seconds.
  while(my_vehicle.current_time < tmax) 
    {
      // Simulate one step
      my_vehicle.simulate();

      // Save data to file
      fprintf(fp,"%f %f %f %f %f %f %f\n", my_vehicle.current_time,
	      my_vehicle.plant_state[0], my_vehicle.plant_state[1],
	      my_vehicle.plant_state[2], my_vehicle.ctrl_input[0],
	      my_vehicle.ctrl_input[1], my_vehicle.ctrl_input[2]);
    }

  fclose(fp);

  for(ii = 0; ii < my_vehicle.fuzzy_ts_sys->get_num_of_rules(); ii++)
    {
      delete [] ts_coeff[ii];
    }
  delete [] ts_coeff;
}



